@@ -56,10 +56,10 @@ except ImportError:
 # UDIAG_SHOW_RQLEN,
 # UDIAG_SHOW_MEMINFO
 
+RUN_AS_MODULE = False
+
 
 class UserCtxtMap(Mapping):
-    _data = {}
-
     _sk_inode_re = re.compile(r"socket:\[(?P<ino>\d+)\]")
 
     _proc_sk_fd_cast = "/proc/%d/fd/%d"
@@ -71,7 +71,7 @@ class UserCtxtMap(Mapping):
         inode = None
 
         sk_inode_raw = os.readlink(sk_path)
-        inode = self._sk_inode_re.search(sk_inode_raw).group('ino')
+        inode = self._sk_inode_re.search(sk_inode_raw).group("ino")
 
         if not inode:
             raise RuntimeError("Unexpected kernel sk inode outline")
@@ -141,14 +141,26 @@ class UserCtxtMap(Mapping):
 
     def _build(self):
         for flow in psutil.net_connections(kind="all"):
-            proc = psutil.Process(flow.pid)
-            usr = proc.username()
+            try:
+                proc = psutil.Process(flow.pid)
+                usr = proc.username()
 
-            ctxt = {"cmd": proc.exe(), "full_cmd": proc.cmdline(), "fds": []}
+                ctxt = {
+                    "cmd": proc.exe(),
+                    "full_cmd": proc.cmdline(),
+                    "fds": [],
+                }
 
-            self._enter_item(usr, flow, ctxt)
+                self._enter_item(usr, flow, ctxt)
+            except (FileNotFoundError, AttributeError, psutil.NoSuchProcess):
+                # Handling edge case of race condition between build and parse
+                # time. That's for very volatile, shortlived flows that can
+                # exist during build but are gone once we want to parse the
+                # inode.
+                pass
 
     def __init__(self):
+        self._data = {}
         self._build()
 
     def __getitem__(self, key):
@@ -176,21 +188,21 @@ class Protocol(Callable):
                 # gracefully
                 return None
 
-    def __init__(self, sk_states, fmt='json'):
+    def __init__(self, sk_states, fmt="json"):
         self._states = sk_states
 
         fmter = "_fmt_%s" % fmt
         self._fmt = getattr(self, fmter, None)
 
         def __call__(self, nl_diag_sk, args, usr_ctxt):
-            raise RuntimeError('not implemented')
+            raise RuntimeError("not implemented")
 
     def _fmt_json(self, refined_stats):
         return json.dumps(refined_stats, indent=4)
 
 
 class UNIX(Protocol):
-    def __init__(self, sk_states=SS_CONN, _fmt='json'):
+    def __init__(self, sk_states=SS_CONN, _fmt="json"):
         super(UNIX, self).__init__(sk_states, fmt=_fmt)
 
     def __call__(self, nl_diag_sk, args, usr_ctxt):
@@ -200,17 +212,16 @@ class UNIX(Protocol):
             show=(UDIAG_SHOW_NAME | UDIAG_SHOW_VFS | UDIAG_SHOW_PEER),
         )
         refined_stats = self._refine_diag_raw(sstats, usr_ctxt)
-        printable = self._fmt(refined_stats)
 
-        print(printable)
+        return refined_stats
 
     def _refine_diag_raw(self, raw_stats, usr_ctxt):
-        refined = {'UNIX': {'flows': []}}
+        refined = {"UNIX": {"flows": []}}
 
         def vfs_cb(raw_val):
             out = {}
-            out['inode'] = raw_val['udiag_vfs_ino']
-            out['dev'] = raw_val['udiag_vfs_dev']
+            out["inode"] = raw_val["udiag_vfs_ino"]
+            out["dev"] = raw_val["udiag_vfs_dev"]
 
             return out
 
@@ -219,17 +230,17 @@ class UNIX(Protocol):
         cb_idx = 1
 
         idiag_attr_refine_map = {
-            'UNIX_DIAG_NAME': ('path_name', None),
-            'UNIX_DIAG_VFS': ('vfs', vfs_cb),
-            'UNIX_DIAG_PEER': ('peer_inode', None),
-            'UNIX_DIAG_SHUTDOWN': ('shutdown', None),
+            "UNIX_DIAG_NAME": ("path_name", None),
+            "UNIX_DIAG_VFS": ("vfs", vfs_cb),
+            "UNIX_DIAG_PEER": ("peer_inode", None),
+            "UNIX_DIAG_SHUTDOWN": ("shutdown", None),
         }
 
         for raw_flow in raw_stats:
             vessel = {}
-            vessel['inode'] = raw_flow['udiag_ino']
+            vessel["inode"] = raw_flow["udiag_ino"]
 
-            for attr in raw_flow['attrs']:
+            for attr in raw_flow["attrs"]:
                 attr_k = attr[k_idx]
                 attr_val = attr[val_idx]
                 k = idiag_attr_refine_map[attr_k][k_idx]
@@ -240,13 +251,13 @@ class UNIX(Protocol):
 
                 vessel[k] = attr_val
 
-            refined['UNIX']['flows'].append(vessel)
+            refined["UNIX"]["flows"].append(vessel)
 
         if usr_ctxt:
-            for flow in refined['UNIX']['flows']:
+            for flow in refined["UNIX"]["flows"]:
                 try:
-                    sk_inode = flow['inode']
-                    flow['usr_ctxt'] = usr_ctxt[sk_inode]
+                    sk_inode = flow["inode"]
+                    flow["usr_ctxt"] = usr_ctxt[sk_inode]
                 except KeyError:
                     # might define sentinel val
                     pass
@@ -260,7 +271,7 @@ class TCP(Protocol):
     INET_DIAG_VEGASINFO = 3
     INET_DIAG_CONG = 4
 
-    def __init__(self, sk_states=SS_CONN, _fmt='json'):
+    def __init__(self, sk_states=SS_CONN, _fmt="json"):
         super(TCP, self).__init__(sk_states, fmt=_fmt)
 
         IDIAG_EXT_FLAGS = [
@@ -279,21 +290,20 @@ class TCP(Protocol):
             states=self._states, family=AF_INET, extensions=self.ext_f
         )
         refined_stats = self._refine_diag_raw(sstats, args.resolve, usr_ctxt)
-        printable = self._fmt(refined_stats)
 
-        print(printable)
+        return refined_stats
 
     def _refine_diag_raw(self, raw_stats, do_resolve, usr_ctxt):
-        refined = {'TCP': {'flows': []}}
+        refined = {"TCP": {"flows": []}}
 
         idiag_refine_map = {
-            'src': 'idiag_src',
-            'dst': 'idiag_dst',
-            'src_port': 'idiag_sport',
-            'dst_port': 'idiag_dport',
-            'inode': 'idiag_inode',
-            'iface_idx': 'idiag_if',
-            'retrans': 'idiag_retrans',
+            "src": "idiag_src",
+            "dst": "idiag_dst",
+            "src_port": "idiag_sport",
+            "dst_port": "idiag_dport",
+            "inode": "idiag_inode",
+            "iface_idx": "idiag_if",
+            "retrans": "idiag_retrans",
         }
 
         for raw_flow in raw_stats:
@@ -301,56 +311,56 @@ class TCP(Protocol):
             for k1, k2 in idiag_refine_map.items():
                 vessel[k1] = raw_flow[k2]
 
-            for ext_bundle in raw_flow['attrs']:
+            for ext_bundle in raw_flow["attrs"]:
                 vessel = self._refine_extension(vessel, ext_bundle)
 
-            refined['TCP']['flows'].append(vessel)
+            refined["TCP"]["flows"].append(vessel)
 
         if usr_ctxt:
-            for flow in refined['TCP']['flows']:
+            for flow in refined["TCP"]["flows"]:
                 try:
-                    sk_inode = flow['inode']
-                    flow['usr_ctxt'] = usr_ctxt[sk_inode]
+                    sk_inode = flow["inode"]
+                    flow["usr_ctxt"] = usr_ctxt[sk_inode]
                 except KeyError:
                     # might define sentinel val
                     pass
 
         if do_resolve:
-            for flow in refined['TCP']['flows']:
-                src_host = Protocol.Resolver.getHost(flow['src'])
+            for flow in refined["TCP"]["flows"]:
+                src_host = Protocol.Resolver.getHost(flow["src"])
                 if src_host:
-                    flow['src_host'] = src_host
+                    flow["src_host"] = src_host
 
-                dst_host = Protocol.Resolver.getHost(flow['dst'])
+                dst_host = Protocol.Resolver.getHost(flow["dst"])
                 if dst_host:
-                    flow['dst_host'] = dst_host
+                    flow["dst_host"] = dst_host
 
         return refined
 
     def _refine_extension(self, vessel, raw_ext):
         k, content = raw_ext
         ext_refine_map = {
-            'meminfo': {
-                'r': 'idiag_rmem',
-                'w': 'idiag_wmem',
-                'f': 'idiag_fmem',
-                't': 'idiag_tmem',
+            "meminfo": {
+                "r": "idiag_rmem",
+                "w": "idiag_wmem",
+                "f": "idiag_fmem",
+                "t": "idiag_tmem",
             }
         }
 
-        if k == 'INET_DIAG_MEMINFO':
-            mem_k = 'meminfo'
+        if k == "INET_DIAG_MEMINFO":
+            mem_k = "meminfo"
             vessel[mem_k] = {}
             for k1, k2 in ext_refine_map[mem_k].items():
                 vessel[mem_k][k1] = content[k2]
 
-        elif k == 'INET_DIAG_CONG':
-            vessel['cong_algo'] = content
+        elif k == "INET_DIAG_CONG":
+            vessel["cong_algo"] = content
 
-        elif k == 'INET_DIAG_INFO':
+        elif k == "INET_DIAG_INFO":
             vessel = self._refine_tcp_info(vessel, content)
 
-        elif k == 'INET_DIAG_SHUTDOWN':
+        elif k == "INET_DIAG_SHUTDOWN":
             pass
 
         return vessel
@@ -381,19 +391,19 @@ class TCP(Protocol):
 
         @staticmethod
         def rtt_p_cb(key, value, **ctx):
-            tcp_info_raw = ctx['raw']
+            tcp_info_raw = ctx["raw"]
 
             try:
                 if (
-                    tcp_info_raw['tcpv_enabled'] != 0
-                    and tcp_info_raw['tcpv_rtt'] != 0x7FFFFFFF
+                    tcp_info_raw["tcpv_enabled"] != 0
+                    and tcp_info_raw["tcpv_rtt"] != 0x7FFFFFFF
                 ):
-                    return tcp_info_raw['tcpv_rtt']
+                    return tcp_info_raw["tcpv_rtt"]
             except KeyError:
                 # ill practice, yet except quicker path
                 pass
 
-            return tcp_info_raw['tcpi_rtt'] / 1000.0
+            return tcp_info_raw["tcpi_rtt"] / 1000.0
 
         # converter
         @staticmethod
@@ -416,7 +426,7 @@ class TCP(Protocol):
 
         @staticmethod
         def opts_c_cb(key, value, **ctx):
-            tcp_info_raw = ctx['raw']
+            tcp_info_raw = ctx["raw"]
 
             # tcp_info opt flags
             TCPI_OPT_TIMESTAMPS = 1
@@ -425,7 +435,7 @@ class TCP(Protocol):
 
             out = []
 
-            opts = tcp_info_raw['tcpi_options']
+            opts = tcp_info_raw["tcpi_options"]
             if opts & TCPI_OPT_TIMESTAMPS:
                 out.append("ts")
             if opts & TCPI_OPT_SACK:
@@ -439,82 +449,82 @@ class TCP(Protocol):
         ti = TCP.InfoCbCore
 
         info_refine_tabl = {
-            'tcpi_state': ('state', ti.state_c_cb),
-            'tcpi_pmtu': ('pmtu', None),
-            'tcpi_retrans': ('retrans', None),
-            'tcpi_ato': ('ato', ti.generic_1k_n_cb),
-            'tcpi_rto': ('rto', ti.rto_n_cb),
+            "tcpi_state": ("state", ti.state_c_cb),
+            "tcpi_pmtu": ("pmtu", None),
+            "tcpi_retrans": ("retrans", None),
+            "tcpi_ato": ("ato", ti.generic_1k_n_cb),
+            "tcpi_rto": ("rto", ti.rto_n_cb),
             # TODO consider wscale baking
-            'tcpi_snd_wscale': ('snd_wscale', None),
-            'tcpi_rcv_wscale': ('rcv_wscale', None),
+            "tcpi_snd_wscale": ("snd_wscale", None),
+            "tcpi_rcv_wscale": ("rcv_wscale", None),
             # TODO bps baking
-            'tcpi_snd_mss': ('snd_mss', None),
-            'tcpi_snd_cwnd': ('snd_cwnd', None),
-            'tcpi_snd_ssthresh': ('snd_ssthresh', ti.snd_thresh_p_cb),
+            "tcpi_snd_mss": ("snd_mss", None),
+            "tcpi_snd_cwnd": ("snd_cwnd", None),
+            "tcpi_snd_ssthresh": ("snd_ssthresh", ti.snd_thresh_p_cb),
             # TODO consider rtt agglomeration - needs nesting
-            'tcpi_rtt': ('rtt', ti.rtt_p_cb),
-            'tcpi_rttvar': ('rttvar', ti.generic_1k_n_cb),
-            'tcpi_rcv_rtt': ('rcv_rtt', ti.generic_1k_n_cb),
-            'tcpi_rcv_space': ('rcv_space', None),
-            'tcpi_options': ('opts', ti.opts_c_cb),
+            "tcpi_rtt": ("rtt", ti.rtt_p_cb),
+            "tcpi_rttvar": ("rttvar", ti.generic_1k_n_cb),
+            "tcpi_rcv_rtt": ("rcv_rtt", ti.generic_1k_n_cb),
+            "tcpi_rcv_space": ("rcv_space", None),
+            "tcpi_options": ("opts", ti.opts_c_cb),
             # unclear, NB not in use by iproute2 ss latest
-            'tcpi_last_data_sent': ('last_data_sent', None),
-            'tcpi_rcv_ssthresh': ('rcv_ssthresh', None),
-            'tcpi_rcv_ssthresh': ('rcv_ssthresh', None),
-            'tcpi_segs_in': ('segs_in', None),
-            'tcpi_segs_out': ('segs_out', None),
-            'tcpi_data_segs_in': ('data_segs_in', None),
-            'tcpi_data_segs_out': ('data_segs_out', None),
-            'tcpi_lost': ('lost', None),
-            'tcpi_notsent_bytes': ('notsent_bytes', None),
-            'tcpi_rcv_mss': ('rcv_mss', None),
-            'tcpi_pacing_rate': ('pacing_rate', None),
-            'tcpi_retransmits': ('retransmits', None),
-            'tcpi_min_rtt': ('min_rtt', None),
-            'tcpi_rwnd_limited': ('rwnd_limited', None),
-            'tcpi_max_pacing_rate': ('max_pacing_rate', None),
-            'tcpi_probes': ('probes', None),
-            'tcpi_reordering': ('reordering', None),
-            'tcpi_last_data_recv': ('last_data_recv', None),
-            'tcpi_bytes_received': ('bytes_received', None),
-            'tcpi_fackets': ('fackets', None),
-            'tcpi_last_ack_recv': ('last_ack_recv', None),
-            'tcpi_last_ack_sent': ('last_ack_sent', None),
-            'tcpi_unacked': ('unacked', None),
-            'tcpi_sacked': ('sacked', None),
-            'tcpi_bytes_acked': ('bytes_acked', None),
-            'tcpi_delivery_rate_app_limited': (
-                'delivery_rate_app_limited',
+            "tcpi_last_data_sent": ("last_data_sent", None),
+            "tcpi_rcv_ssthresh": ("rcv_ssthresh", None),
+            "tcpi_rcv_ssthresh": ("rcv_ssthresh", None),
+            "tcpi_segs_in": ("segs_in", None),
+            "tcpi_segs_out": ("segs_out", None),
+            "tcpi_data_segs_in": ("data_segs_in", None),
+            "tcpi_data_segs_out": ("data_segs_out", None),
+            "tcpi_lost": ("lost", None),
+            "tcpi_notsent_bytes": ("notsent_bytes", None),
+            "tcpi_rcv_mss": ("rcv_mss", None),
+            "tcpi_pacing_rate": ("pacing_rate", None),
+            "tcpi_retransmits": ("retransmits", None),
+            "tcpi_min_rtt": ("min_rtt", None),
+            "tcpi_rwnd_limited": ("rwnd_limited", None),
+            "tcpi_max_pacing_rate": ("max_pacing_rate", None),
+            "tcpi_probes": ("probes", None),
+            "tcpi_reordering": ("reordering", None),
+            "tcpi_last_data_recv": ("last_data_recv", None),
+            "tcpi_bytes_received": ("bytes_received", None),
+            "tcpi_fackets": ("fackets", None),
+            "tcpi_last_ack_recv": ("last_ack_recv", None),
+            "tcpi_last_ack_sent": ("last_ack_sent", None),
+            "tcpi_unacked": ("unacked", None),
+            "tcpi_sacked": ("sacked", None),
+            "tcpi_bytes_acked": ("bytes_acked", None),
+            "tcpi_delivery_rate_app_limited": (
+                "delivery_rate_app_limited",
                 None,
             ),
-            'tcpi_delivery_rate': ('delivery_rate', None),
-            'tcpi_sndbuf_limited': ('sndbuf_limited', None),
-            'tcpi_ca_state': ('ca_state', None),
-            'tcpi_busy_time': ('busy_time', None),
-            'tcpi_total_retrans': ('total_retrans', None),
-            'tcpi_advmss': ('advmss', None),
-            'tcpi_backoff': (None, None),
-            'tcpv_enabled': (None, 'skip'),
-            'tcpv_rttcnt': (None, 'skip'),
-            'tcpv_rtt': (None, 'skip'),
-            'tcpv_minrtt': (None, 'skip'),
+            "tcpi_delivery_rate": ("delivery_rate", None),
+            "tcpi_sndbuf_limited": ("sndbuf_limited", None),
+            "tcpi_ca_state": ("ca_state", None),
+            "tcpi_busy_time": ("busy_time", None),
+            "tcpi_total_retrans": ("total_retrans", None),
+            "tcpi_advmss": ("advmss", None),
+            "tcpi_backoff": (None, None),
+            "tcpv_enabled": (None, "skip"),
+            "tcpv_rttcnt": (None, "skip"),
+            "tcpv_rtt": (None, "skip"),
+            "tcpv_minrtt": (None, "skip"),
             # BBR
-            'bbr_bw_lo': ('bbr_bw_lo', None),
-            'bbr_bw_hi': ('bbr_bw_hi', None),
-            'bbr_min_rtt': ('bbr_min_rtt', None),
-            'bbr_pacing_gain': ('bbr_pacing_gain', None),
-            'bbr_cwnd_gain': ('bbr_cwnd_gain', None),
+            "bbr_bw_lo": ("bbr_bw_lo", None),
+            "bbr_bw_hi": ("bbr_bw_hi", None),
+            "bbr_min_rtt": ("bbr_min_rtt", None),
+            "bbr_pacing_gain": ("bbr_pacing_gain", None),
+            "bbr_cwnd_gain": ("bbr_cwnd_gain", None),
             # DCTCP
-            'dctcp_enabled': ('dctcp_enabled', None),
-            'dctcp_ce_state': ('dctcp_ce_state', None),
-            'dctcp_alpha': ('dctcp_alpha', None),
-            'dctcp_ab_ecn': ('dctcp_ab_ecn', None),
-            'dctcp_ab_tot': ('dctcp_ab_tot', None),
+            "dctcp_enabled": ("dctcp_enabled", None),
+            "dctcp_ce_state": ("dctcp_ce_state", None),
+            "dctcp_alpha": ("dctcp_alpha", None),
+            "dctcp_ab_ecn": ("dctcp_ab_ecn", None),
+            "dctcp_ab_tot": ("dctcp_ab_tot", None),
         }
         k_idx = 0
         cb_idx = 1
 
-        info_k = 'tcp_info'
+        info_k = "tcp_info"
         vessel[info_k] = {}
 
         # BUG - pyroute2 diag - seems always last info instance from kernel
@@ -525,10 +535,10 @@ class TCP(Protocol):
                 refined_k = info_refine_tabl[k][k_idx]
                 cb = info_refine_tabl[k][cb_idx]
                 refined_v = v
-                if cb and cb == 'skip':
+                if cb and cb == "skip":
                     continue
                 elif cb:
-                    ctx = {'raw': tcp_info_raw}
+                    ctx = {"raw": tcp_info_raw}
                     refined_v = cb(k, v, **ctx)
 
                 vessel[info_k][refined_k] = refined_v
@@ -544,34 +554,34 @@ def prepare_args():
                                      iproute2/misc/ss2"""
     )
     parser.add_argument(
-        '-x',
-        '--unix',
-        help='Display Unix domain sockets.',
-        action='store_true',
+        "-x",
+        "--unix",
+        help="Display Unix domain sockets.",
+        action="store_true",
     )
     parser.add_argument(
-        '-t', '--tcp', help='Display TCP sockets.', action='store_true'
+        "-t", "--tcp", help="Display TCP sockets.", action="store_true"
     )
     parser.add_argument(
-        '-l',
-        '--listen',
-        help='Display listening sockets.',
-        action='store_true',
+        "-l",
+        "--listen",
+        help="Display listening sockets.",
+        action="store_true",
     )
     parser.add_argument(
-        '-a', '--all', help='Display all sockets.', action='store_true'
+        "-a", "--all", help="Display all sockets.", action="store_true"
     )
     parser.add_argument(
-        '-p',
-        '--process',
-        help='show socket holding context',
-        action='store_true',
+        "-p",
+        "--process",
+        help="show socket holding context",
+        action="store_true",
     )
     parser.add_argument(
-        '-r',
-        '--resolve',
-        help='resolve host names in addition',
-        action='store_true',
+        "-r",
+        "--resolve",
+        help="resolve host names in addition",
+        action="store_true",
     )
 
     args = parser.parse_args()
@@ -581,7 +591,7 @@ def prepare_args():
 
 def run(args=None):
     if psutil is None:
-        raise RuntimeError('ss2 requires python-psutil >= 5.0 to run')
+        raise RuntimeError("ss2 requires python-psutil >= 5.0 to run")
 
     if not args:
         args = prepare_args()
@@ -600,16 +610,24 @@ def run(args=None):
         protocols.append(UNIX(sk_states=_states))
 
     if not protocols:
-        raise RuntimeError('not implemented - ss2 in fledging mode')
+        raise RuntimeError("not implemented - ss2 in fledging mode")
 
     _user_ctxt_map = None
     if args.process:
         _user_ctxt_map = UserCtxtMap()
 
+    result = list()
+
     with DiagSocket() as ds:
         ds.bind()
         for p in protocols:
-            p(ds, args, _user_ctxt_map)
+            sub_statistics = p(ds, args, _user_ctxt_map)
+            result.append(sub_statistics)
+
+    if RUN_AS_MODULE:
+        return result
+    else:
+        print(json.dumps(result, indent=4))
 
 
 if __name__ == "__main__":
