@@ -1,6 +1,6 @@
 '''
-Netns management overview
-=========================
+Basic network namespace management
+==================================
 
 Pyroute2 provides basic namespaces management support.
 Here's a quick overview of typical netns tasks and
@@ -63,8 +63,8 @@ Spawn a process within a netns
 For that purpose one can use `NSPopen` API. It works just
 as normal `Popen`, but starts a process within a netns.
 
-List, set, create and remove netns
-----------------------------------
+List, set, create, attach and remove netns
+------------------------------------------
 
 These functions are described below. To use them, import
 `netns` module::
@@ -79,33 +79,38 @@ loading this module, dumps the core, one can check the
 SELinux state with `getenforce` command.
 
 '''
+import ctypes
+import ctypes.util
+import errno
 import io
 import os
 import os.path
-import errno
-import ctypes
-import ctypes.util
 import pickle
 import struct
 import traceback
+
 from pyroute2 import config
 from pyroute2.common import basestring
+
 try:
     file = file
 except NameError:
     file = io.IOBase
 
 # FIXME: arch reference
-__NR = {'x86_': {'64bit': 308},
-        'i386': {'32bit': 346},
-        'i686': {'32bit': 346},
-        'mips': {'32bit': 4344,
-                 '64bit': 5303},  # FIXME: NABI32?
-        'armv': {'32bit': 375},
-        'aarc': {'32bit': 375,
-                 '64bit': 268},  # FIXME: EABI vs. OABI?
-        'ppc6': {'64bit': 350},
-        's390': {'64bit': 339}}
+__NR = {
+    'x86_': {'64bit': 308},
+    'i386': {'32bit': 346},
+    'i686': {'32bit': 346},
+    'mips': {'32bit': 4344, '64bit': 5303},  # FIXME: NABI32?
+    'loon': {'64bit': 268},
+    'armv': {'32bit': 375},
+    'aarc': {'32bit': 375, '64bit': 268},  # FIXME: EABI vs. OABI?
+    'ppc6': {'64bit': 350},
+    's390': {'64bit': 339},
+    'loongarch64': {'64bit': 268},
+    'risc': {'64bit': 268},
+}
 __NR_setns = __NR.get(config.machine[:4], {}).get(config.arch, 308)
 
 CLONE_NEWNET = 0x40000000
@@ -116,6 +121,7 @@ MS_SHARED = 1 << 20
 NETNS_RUN_DIR = '/var/run/netns'
 
 __saved_ns = []
+__libc = None
 
 
 def _get_netnspath(name):
@@ -128,6 +134,15 @@ def _get_netnspath(name):
     return netnspath
 
 
+def _get_libc(libc=None):
+    global __libc
+    if libc is not None:
+        return libc
+    if __libc is None:
+        __libc = ctypes.CDLL(ctypes.util.find_library('c'), use_errno=True)
+    return __libc
+
+
 def listnetns(nspath=None):
     '''
     List available network namespaces.
@@ -138,11 +153,8 @@ def listnetns(nspath=None):
         nsdir = NETNS_RUN_DIR
     try:
         return os.listdir(nsdir)
-    except OSError as e:
-        if e.errno == errno.ENOENT:
-            return []
-        else:
-            raise
+    except FileNotFoundError:
+        return []
 
 
 def _get_ns_by_inode(nspath=NETNS_RUN_DIR):
@@ -153,7 +165,11 @@ def _get_ns_by_inode(nspath=NETNS_RUN_DIR):
     ns_by_dev_inode = {}
     for ns_name in listnetns(nspath=nspath):
         ns_path = os.path.join(nspath, ns_name)
-        st = os.stat(ns_path)
+        try:
+            st = os.stat(ns_path)
+        except FileNotFoundError:
+            # The path disappeared from the FS while listing, ignore it
+            continue
         if st.st_dev not in ns_by_dev_inode:
             ns_by_dev_inode[st.st_dev] = {}
         ns_by_dev_inode[st.st_dev][st.st_ino] = ns_name
@@ -206,8 +222,8 @@ def pid_to_ns(pid=1, nspath=NETNS_RUN_DIR):
         return None
 
 
-def _create(netns, libc=None):
-    libc = libc or ctypes.CDLL(ctypes.util.find_library('c'), use_errno=True)
+def _create(netns, libc=None, pid=None):
+    libc = _get_libc(libc)
     netnspath = _get_netnspath(netns)
     netnsdir = os.path.dirname(netnspath)
 
@@ -223,8 +239,10 @@ def _create(netns, libc=None):
     while libc.mount(b'', netnsdir, b'none', MS_SHARED | MS_REC, None) != 0:
         if done:
             raise OSError(ctypes.get_errno(), 'share rundir failed', netns)
-        if libc.mount(netnsdir, netnsdir, b'none', MS_BIND | MS_REC,
-                      None) != 0:
+        if (
+            libc.mount(netnsdir, netnsdir, b'none', MS_BIND | MS_REC, None)
+            != 0
+        ):
             raise OSError(ctypes.get_errno(), 'mount rundir failed', netns)
         done = True
 
@@ -232,11 +250,22 @@ def _create(netns, libc=None):
     os.close(os.open(netnspath, os.O_RDONLY | os.O_CREAT | os.O_EXCL, 0))
 
     # unshare
-    if libc.unshare(CLONE_NEWNET) < 0:
-        raise OSError(ctypes.get_errno(), 'unshare failed', netns)
+    if pid is None:
+        pid = 'self'
+        if libc.unshare(CLONE_NEWNET) < 0:
+            raise OSError(ctypes.get_errno(), 'unshare failed', netns)
 
     # bind the namespace
-    if libc.mount(b'/proc/self/ns/net', netnspath, b'none', MS_BIND, None) < 0:
+    if (
+        libc.mount(
+            '/proc/{}/ns/net'.format(pid).encode('utf-8'),
+            netnspath,
+            b'none',
+            MS_BIND,
+            None,
+        )
+        < 0
+    ):
         raise OSError(ctypes.get_errno(), 'mount failed', netns)
 
 
@@ -269,11 +298,19 @@ def create(netns, libc=None):
             raise error
 
 
+def attach(netns, pid, libc=None):
+    '''
+    Attach the network namespace of the process `pid`
+    to `netns` as if it were created with `create`.
+    '''
+    _create(netns, libc, pid)
+
+
 def remove(netns, libc=None):
     '''
     Remove a network namespace.
     '''
-    libc = libc or ctypes.CDLL(ctypes.util.find_library('c'), use_errno=True)
+    libc = _get_libc(libc)
     netnspath = _get_netnspath(netns)
     libc.umount2(netnspath, MNT_DETACH)
     os.unlink(netnspath)
@@ -300,7 +337,7 @@ def setns(netns, flags=os.O_CREAT, libc=None):
     not provided via arguments.
     '''
     newfd = False
-    libc = libc or ctypes.CDLL(ctypes.util.find_library('c'), use_errno=True)
+    libc = _get_libc(libc)
     if isinstance(netns, basestring):
         netnspath = _get_netnspath(netns)
         if os.path.basename(netns) in listnetns(os.path.dirname(netns)):
